from cymysql.tests.test_issues import * # noqa
from cymysql.tests.test_example import * # noqa
from cymysql.tests.test_basic import * # noqa
from cymysql.tests.test_DictCursor import * # noqa

if __name__ == "__main__":
    import unittest
    unittest.main()
